package com.ss.android.article.wenda.digg

import android.content.Context
import android.content.res.ColorStateList
import android.graphics.Typeface
import android.support.annotation.StringRes
import android.util.AttributeSet
import android.util.TypedValue
import android.view.Gravity
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import com.airbnb.lottie.LottieAnimationView
import com.bytedance.article.common.utils.ViewUtils
import com.bytedance.common.utility.UIUtils
import com.ss.android.article.base.R

/**
 * 新DiggLayout，点赞动画使用Lottie
 *
 * @author zhanghao
 * @date 2018/5/9
 * @email zhanghao@bytedance.com
 */
class NewDiggLayout : LinearLayout {

    private lateinit var mAnimationView: LottieAnimationView
    private lateinit var mDiggTv: TextView

    /** attrs **/
    private var mDiggType = DIGG_TYPE_UNSPECIFIED
    private var mDiggTextSize = UIUtils.sp2px(context, 14f).toInt()
    private var mDiggTextColor: ColorStateList? = context.resources.getColorStateList(R.color.c8_c3_selector)
    private var mDrawablePadding = 0
    private var mDrawableWidth = UIUtils.dip2Px(context, 32f).toInt()
    private var mDrawableHeight = UIUtils.dip2Px(context, 32f).toInt()
    private var mShowDiggText = true

    /** Digital：数字状态， Word：文字状态，这两种状态可能有不同的字号，字体等 **/
    private var mDigitalTextSize = UIUtils.sp2px(context, 14f).toInt()
    private var mWordTextSize = UIUtils.sp2px(context, 14f).toInt()

    private var mIsDigg = false

    companion object {
        const val DIGG_TYPE_UNSPECIFIED = -1
        const val DIGG_TYPE_FEED = 0
        const val DIGG_TYPE_DETAIL = 1
    }

    constructor(context: Context) : this(context, null)

    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)

    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        init(context, attrs, defStyleAttr)
    }

    private fun init(context: Context, attrs: AttributeSet?, defStyleAttr: Int) {
        //允许动画超出边界
        clipToPadding = false
        clipChildren = false
        orientation = HORIZONTAL
        gravity = Gravity.CENTER_VERTICAL

        if (attrs != null) {
            val typedArray = context.obtainStyledAttributes(attrs, R.styleable.NewDiggLayout, defStyleAttr, 0)
            mDiggType = typedArray.getInt(R.styleable.NewDiggLayout_diggType, DIGG_TYPE_UNSPECIFIED)
            initByDiggType()
            val mDiggTextSize = typedArray.getDimension(R.styleable.NewDiggLayout_diggTextSize, 0f).toInt()
            if (mDiggTextSize > 0) {
                mDigitalTextSize = mDiggTextSize
                mWordTextSize = mDiggTextSize
            }
            val diggTextColor = typedArray.getColorStateList(R.styleable.NewDiggLayout_diggTextColor)
            if (diggTextColor != null) {
                mDiggTextColor = diggTextColor
            }
            mDrawablePadding = typedArray.getDimension(R.styleable.NewDiggLayout_diggDrawablePadding, mDrawablePadding.toFloat()).toInt()
            mDrawableWidth = typedArray.getDimension(R.styleable.NewDiggLayout_diggDrawableWidth, mDrawableWidth.toFloat()).toInt()
            mDrawableHeight = typedArray.getDimension(R.styleable.NewDiggLayout_diggDrawableHeight, mDrawableHeight.toFloat()).toInt()
            mShowDiggText = typedArray.getBoolean(R.styleable.NewDiggLayout_diggShowDiggText, mShowDiggText)
            typedArray.recycle()
        }

        val drawableLayoutParams = ViewGroup.MarginLayoutParams(mDrawableWidth, mDrawableHeight)
        mAnimationView = LottieAnimationView(context)
        mAnimationView.setAnimation("like.json", LottieAnimationView.CacheStrategy.Weak)
        addView(mAnimationView, drawableLayoutParams)

        val diggTvLayoutParams = ViewGroup.MarginLayoutParams(ViewGroup.MarginLayoutParams.WRAP_CONTENT, ViewGroup.MarginLayoutParams.WRAP_CONTENT)
        mDiggTv = TextView(context)
        mDiggTv.setTextSize(TypedValue.COMPLEX_UNIT_PX, mDiggTextSize.toFloat())
        mDiggTv.setTextColor(mDiggTextColor)
        addView(mDiggTv, diggTvLayoutParams)
        setDrawablePadding(mDrawablePadding)
        mDiggTv.visibility = if(mShowDiggText) VISIBLE else GONE
    }

    /**
     * 点击点赞，做动画
     */
    fun onDiggClick(isDigg: Boolean) {
        if (mIsDigg == isDigg) {
            return
        }
        mIsDigg = isDigg
        mDiggTv.isSelected = isDigg
        mAnimationView.cancelAnimation()
        mAnimationView.progress = 0f
        if (isDigg) {
            mAnimationView.playAnimation()
        }
    }

    /**
     * 设置点赞状态，不做动画
     */
    fun setDiggStatus(isDigg: Boolean) {
        mIsDigg = isDigg
        mDiggTv.isSelected = isDigg
        mAnimationView.cancelAnimation()
        mAnimationView.progress = if (isDigg) 1f else 0f
    }

    fun setDiggCount(diggCount: Int) {
        ViewUtils.setTextWithDINStyle(mDiggTv, ViewUtils.getDiggText(diggCount), mDigitalTextSize, mWordTextSize)
    }

    fun setText(text: CharSequence?) {
        mDiggTv.text = text
    }

    fun setText(@StringRes resId: Int) {
        mDiggTv.setText(resId)
    }

    fun setTextSize(size: Int) {
        mDiggTextSize = size
        mDiggTv.textSize = size.toFloat()
        requestLayout()
    }

    fun setTextColor(color: ColorStateList) {
        mDiggTextColor = color
        mDiggTv.setTextColor(color)
    }

    fun setFakeBoldText(isFakeBoldText: Boolean) {
        mDiggTv.paint.isFakeBoldText = isFakeBoldText
        mDiggTv.invalidate()
    }

    fun setDrawablePadding(padding: Int) {
        mDrawablePadding = padding
        val diggTvLayoutParams = mDiggTv.layoutParams as ViewGroup.MarginLayoutParams
        diggTvLayoutParams.leftMargin = padding
        mDiggTv.layoutParams = diggTvLayoutParams
        requestLayout()
    }

    fun setDrawableWidth(width: Int) {
        mDrawableWidth = width
        val drawableLayoutParams = mAnimationView.layoutParams as ViewGroup.MarginLayoutParams
        drawableLayoutParams.width = width
        mAnimationView.layoutParams = drawableLayoutParams
    }

    private fun initByDiggType() {
        when (mDiggType) {
            DIGG_TYPE_FEED -> initByConfig(NewDiggConfig.configFeed(context))
            DIGG_TYPE_DETAIL -> initByConfig(NewDiggConfig.configDetail(context))
        }
    }

    private fun initByConfig(config: NewDiggConfig) {
        mDiggTextColor = config.diggTextColor
        mDrawablePadding = config.drawablePadding
        mDrawableWidth = config.drawableWidth
        mDrawableHeight = config.drawableHeight
        mDigitalTextSize = config.digitalTextSize
        mWordTextSize = config.wordTextSize
        mShowDiggText = config.showDiggText
    }
}