package com.ss.android.article.wenda.recommend

import android.support.v4.view.animation.PathInterpolatorCompat
import android.view.View
import android.view.animation.DecelerateInterpolator
import android.view.animation.Interpolator
import com.bytedance.article.common.extensions.ifNotEmpty
import com.bytedance.article.common.extensions.isNotEmpty
import com.bytedance.article.common.extensions.isNullOrEmpty
import com.bytedance.common.utility.UIUtils
import com.bytedance.retrofit2.Call
import com.bytedance.retrofit2.Callback
import com.bytedance.retrofit2.SsResponse
import com.nineoldandroids.animation.*
import com.ss.android.account.utils.DebouncingOnClickListener
import com.ss.android.common.app.AbsApplication
import com.ss.android.wenda.api.entity.SimpleApiResponse
import com.ss.android.wenda.api.entity.answerlist.RecommendFollowData
import com.ss.android.wenda.api.network.WendaApiUtil

/**
 * 推人卡片管理类
 *
 * @author zhanghao
 * @date 2018/4/16
 * @email zhanghao@bytedance.com
 */
class RecommendFollowManager(userId: String?) {

    companion object {
        const val SOURCE_CLICK_ARROW = 0
        const val SOURCE_CLICK_FOLLOW = 1

        const val PULL_UP = 0
        const val PULL_DOWN = 1

        val sInterpolator: Interpolator = PathInterpolatorCompat.create(0.39f, 0.575f, 0.565f, 1f)
    }

    private var mQuestionId: String? = null
    private val mUserId = userId
    private var mApiParams: String? = null

    private var mRecommendDelegate: RecommendFollowDelegate? = null
    private var mArrowView: View? = null

    private var mRecommendData: RecommendFollowData? = null
    private val mRecommendHeight = UIUtils.dip2Px(AbsApplication.getAppContext(), 264f).toInt()
    private var mCurrentAnim: Animator? = null

    var isArrowShown = false

    /** Listeners **/
    private var mPullStartListener: ((clickSource: Int, direction: Int) -> Unit)? = null
    private var mPullEndListener: ((clickSource: Int, direction: Int) -> Unit)? = null
    private var mPullUpdateListener: ((direction: Int, height: Int) -> Unit)? = null
    private var mUserCardFollowListener: ((isFollow: Boolean, userId: String?, position: Int) -> Unit)? = null
    private var mMoreClickListener: (() -> Unit)? = null
    private var mFooterClickListener: (() -> Unit)? = null

    constructor(questionId: String, userId: String?, apiParams: String?) : this(userId) {
        mQuestionId = questionId
        mApiParams = apiParams
    }

    /**
     * 设置推人卡片Delegate
     */
    fun setRecommendDelegate(recommendDelegate: RecommendFollowDelegate) {
        mRecommendDelegate = recommendDelegate
    }

    /**
     * 设置下拉上拉箭头
     */
    fun setArrowView(arrowView: View?) {
        mArrowView = arrowView
        mArrowView?.setOnClickListener(object : DebouncingOnClickListener() {
            override fun doClick(v: View) {
                if (mRecommendDelegate?.recommendLayout?.visibility == View.VISIBLE) {
                    //展开状态
                    pullUpRecommend(SOURCE_CLICK_ARROW)
                } else {
                    //收起状态
                    pullDownRecommend(SOURCE_CLICK_ARROW)
                }
            }
        })
    }

    /**
     * 下拉推人卡片
     */
    fun pullDownRecommend(clickSource: Int) {
        tryInitRecommendLayout()
        val recommendData = mRecommendData
        if (recommendData == null) {
            requestRecommendFollow(clickSource, true)
        } else {
            mRecommendDelegate?.recommendLayout?.tryBind(recommendData)
            mRecommendDelegate?.recommendLayout?.visibility = View.GONE
            doPullDownAnimation(clickSource)
        }
    }

    /**
     * 上拉推人卡片
     */
    fun pullUpRecommend(clickSource: Int) {
        doPullUpAnimation(clickSource)
    }

    /**
     * 直接显示推人卡片，没有动画
     */
    fun showRecommend() {
        tryInitRecommendLayout()
        val recommendData = mRecommendData
        if (recommendData == null) {
            requestRecommendFollow(SOURCE_CLICK_FOLLOW, false)
        } else {
            mRecommendDelegate?.recommendLayout?.tryBind(recommendData)
            mRecommendDelegate?.recommendLayout?.visibility = View.VISIBLE
            mRecommendData?.isPulledDown = true
        }
    }

    /**
     * 直接隐藏推人卡片，没有动画
     */
    fun hideRecommend() {
        mRecommendData?.isPulledDown = false
        mRecommendDelegate?.recommendLayout?.visibility = View.GONE
    }

    /**
     * 请求数据，但是不展示推人卡片
     */
    fun preloadRecommend(preloadResponseListener: () -> Unit) {
        tryInitRecommendLayout()
        WendaApiUtil.recommendFollow(mQuestionId, mUserId, mApiParams, object : Callback<SimpleApiResponse<RecommendFollowData>> {
            override fun onResponse(call: Call<SimpleApiResponse<RecommendFollowData>>?, response: SsResponse<SimpleApiResponse<RecommendFollowData>>?) {
                val data = response?.body()?.data ?: return
                data.reco_follow_users.ifNotEmpty {
                    mRecommendData = data
                    mRecommendDelegate?.recommendLayout?.bind(data)
                    mRecommendDelegate?.recommendLayout?.visibility = View.GONE
                    preloadResponseListener.invoke()
                }
            }

            override fun onFailure(call: Call<SimpleApiResponse<RecommendFollowData>>?, t: Throwable?) {

            }
        })
    }

    /**
     * 上下拉开始回调
     */
    fun setPullStartListener(listener: (clickSource: Int, direction: Int) -> Unit) {
        mPullStartListener = listener
    }

    /**
     * 上下拉结束回调
     */
    fun setPullEndListener(listener: (clickSource: Int, direction: Int) -> Unit) {
        mPullEndListener = listener
    }

    /**
     * 上下拉(过程)更新回调
     */
    fun setPullUpdateListener(listener: (direction: Int, height: Int) -> Unit) {
        mPullUpdateListener = listener
    }

    /**
     * 用户卡片关注点击回调
     */
    fun setUserCardFollowListener(listener: (isFollow: Boolean, userId: String?, position: Int) -> Unit) {
        mUserCardFollowListener = listener
        mUserCardFollowListener?.let {
            mRecommendDelegate?.recommendLayout?.setUserCardFollowListener(it)
        }
    }

    /**
     * 查看更多点击回调
     */
    fun setMoreClickListener(listener: () -> Unit) {
        mMoreClickListener = listener
        mMoreClickListener?.let {
            mRecommendDelegate?.recommendLayout?.setMoreClickListener(it)
        }
    }

    /**
     * Footer点击回调
     */
    fun setFooterClickListener(listener: () -> Unit) {
        mFooterClickListener = listener
        mFooterClickListener?.let {
            mRecommendDelegate?.recommendLayout?.setFooterClickListener(it)
        }
    }

    /**
     * 清除数据
     */
    fun clearData() {
        mRecommendData = null
        releaseResource()
    }

    /**
     * 回收资源
     */
    fun releaseResource() {
        mCurrentAnim?.cancel()
        mRecommendDelegate?.recommendLayout?.releaseResource()
    }

    fun getRecommendData() = mRecommendData

    private fun requestRecommendFollow(clickSource: Int, shouldDoAnimation: Boolean) {
        WendaApiUtil.recommendFollow(mQuestionId, mUserId, mApiParams, object : Callback<SimpleApiResponse<RecommendFollowData>> {
            override fun onResponse(call: Call<SimpleApiResponse<RecommendFollowData>>?, response: SsResponse<SimpleApiResponse<RecommendFollowData>>?) {
                val data = response?.body()?.data ?: return
                data.reco_follow_users.ifNotEmpty {
                    mRecommendData = data
                    mRecommendDelegate?.recommendLayout?.bind(data)
                    if (shouldDoAnimation) {
                        mRecommendDelegate?.recommendLayout?.visibility = View.GONE
                        doPullDownAnimation(clickSource)
                    } else {
                        mRecommendDelegate?.recommendLayout?.visibility = View.VISIBLE
                        mRecommendData?.isPulledDown = true
                    }
                }
            }

            override fun onFailure(call: Call<SimpleApiResponse<RecommendFollowData>>?, t: Throwable?) {

            }
        })
    }

    private fun tryInitRecommendLayout() {
        mRecommendDelegate?.tryInitRecommendLayout(mArrowView)
        mUserCardFollowListener?.let {
            mRecommendDelegate?.recommendLayout?.setUserCardFollowListener(it)
        }
        mMoreClickListener?.let {
            mRecommendDelegate?.recommendLayout?.setMoreClickListener(it)
        }
        mFooterClickListener?.let {
            mRecommendDelegate?.recommendLayout?.setFooterClickListener(it)
        }
    }

    private fun doPullUpAnimation(clickSource: Int) {
        if (mRecommendDelegate?.recommendLayout?.visibility != View.VISIBLE) {
            return
        }
        val recommendAnim = getRecommendAnim(PULL_UP)
        mCurrentAnim?.cancel()
        mCurrentAnim = recommendAnim
        recommendAnim.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationStart(animator: Animator?) {
                super.onAnimationStart(animator)
                mPullStartListener?.invoke(clickSource, PULL_UP)
            }

            override fun onAnimationEnd(animator: Animator?) {
                super.onAnimationEnd(animator)
                mCurrentAnim = null
                mRecommendData?.isPulledDown = false
                mRecommendDelegate?.recommendLayout?.visibility = View.GONE
                mPullEndListener?.invoke(clickSource, PULL_UP)
            }
        })
        recommendAnim.duration = 250
        recommendAnim.start()
    }

    private fun doPullDownAnimation(clickSource: Int) {
        mRecommendDelegate?.tryAdjustTrianglePos(mArrowView)
        if (mRecommendDelegate?.recommendLayout?.visibility != View.GONE) {
            return
        }
        val pullDownAnimatorSet = AnimatorSet()
        mCurrentAnim?.cancel()
        mCurrentAnim = pullDownAnimatorSet
        val alphaAnim: Animator
        alphaAnim = ObjectAnimator.ofFloat(mRecommendDelegate?.recommendLayout, "alpha", 0.2f, 1f)
        alphaAnim.setInterpolator(DecelerateInterpolator())
        val recommendAnim = getRecommendAnim(PULL_DOWN)
        pullDownAnimatorSet.playTogether(alphaAnim, recommendAnim)
        pullDownAnimatorSet.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationStart(animator: Animator?) {
                super.onAnimationStart(animator)
                mRecommendDelegate?.recommendLayout?.visibility = View.VISIBLE
                mPullStartListener?.invoke(clickSource, PULL_DOWN)
            }

            override fun onAnimationEnd(animator: Animator?) {
                super.onAnimationEnd(animator)
                mCurrentAnim = null
                mRecommendData?.isPulledDown = true
                mPullEndListener?.invoke(clickSource, PULL_DOWN)
            }
        })
        pullDownAnimatorSet.duration = 250
        pullDownAnimatorSet.start()
    }

    private fun getRecommendAnim(direction: Int): Animator {
        val from = if (direction == PULL_UP) mRecommendHeight else 0
        val to = if (direction == PULL_UP) 0 else mRecommendHeight
        val animator = ValueAnimator.ofInt(from, to)
        if (mRecommendDelegate?.recommendContainer == null) {
            return animator
        }
        animator.interpolator = sInterpolator
        animator.addUpdateListener { valueAnimator ->
            val layoutParams = mRecommendDelegate?.recommendContainer?.layoutParams
            layoutParams?.height = valueAnimator.animatedValue as Int
            mRecommendDelegate?.recommendContainer?.layoutParams = layoutParams
            mPullUpdateListener?.invoke(direction, valueAnimator.animatedValue as Int)
        }
        return animator
    }
}